from sidemash_sdk.Auth import Auth
from sidemash_sdk.StreamSquareService import StreamSquareService


class Client:
    def __init__(self, auth: Auth):
        self._type = "Client" 
        self.auth = auth
        self._stream_square = StreamSquareService(auth)

    def stream_square(self):
        return self._stream_square

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "Client(auth=" + repr(self.auth) + ")"


from typing import Dict
import json


class SecureAndNonSecure:
    def __init__(self, secure: str, non_secure_on80: str, non_secure: str):
        self._type = "SecureAndNonSecure" 
        self.secure = secure
        self.non_secure_on80 = non_secure_on80
        self.non_secure = non_secure

    @staticmethod
    def _type(): 
        return "SecureAndNonSecure"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return SecureAndNonSecure.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('secure', self.secure),
            ('nonSecureOn80', self.non_secure_on80),
            ('nonSecure', self.non_secure)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('secure', self.secure),
            ('non_secure_on80', self.non_secure_on80),
            ('non_secure', self.non_secure)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return SecureAndNonSecure(d["secure"],
                                  d["nonSecureOn80"],
                                  d["nonSecure"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return SecureAndNonSecure(d["secure"],
                                  d["non_secure_on80"],
                                  d["non_secure"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("SecureAndNonSecure(secure=" + self.secure + 
                                   ", non_secure_on80=" + self.non_secure_on80 + 
                                   ", non_secure=" + self.non_secure + ")")


from abc import ABC, abstractmethod
from sidemash_sdk.HttpMethod import HttpMethod
from typing import Dict
import json


class Hook(ABC):
    def is_http_call(self):
        return self._type == HttpCall._type()

    def is_not_http_call(self):
        return self._type != HttpCall._type()

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Hook.from_dict(value)

    def __to_remote_dict(self):
        pass

    def to_dict(self):
        pass

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        ty = d['_type']
        if ty == HttpCall._type(): return HttpCall.from_dict(d)
        else: raise ValueError("Invalid dict submitted for creating 'Hook'" + " Unexpected '_type' = " + ty)

    @staticmethod
    def from_dict(d: Dict[str, any]):
        ty = d['_type']
        if ty == HttpCall._type(): return HttpCall.from_dict(d)
        else: raise ValueError("Invalid dict submitted for creating 'Hook'" + " Unexpected '_type' = " + ty)

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())



class HttpCall:
    def __init__(self, method: HttpMethod, url: str):
        self._type = "Hook.HttpCall" 
        self.method = method
        self.url = url

    @staticmethod
    def _type(): 
        return "Hook.HttpCall"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return HttpCall.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('method', str(self.method)),
            ('url', self.url)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('method', str(self.method)),
            ('url', self.url)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return HttpCall(HttpMethod.from_string(d["method"]),
                        d["url"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return HttpCall(HttpMethod.from_string(d["method"]),
                        d["url"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("HttpCall(method=" + repr(self.method) + 
                         ", url=" + self.url + ")")


import time


class SdmRequest:

    def __init__(self, signed_headers, method, path, query_string, body_hash):
        self.nonce = int(round(time.time() * 1000))
        self.signed_headers = signed_headers
        self.method = method
        self.path = path
        self.query_string = query_string
        self.body_hash = body_hash

    def to_message(self):
        return (
            str(self.nonce) +
            "".join([name + ":" + value for name, value in list(self.signed_headers.items()) ]) +
            self.method +
            self.path +
            (self.query_string if self.query_string is not None else "") +
            (self.body_hash if self.body_hash is not None else "")
        )



from typing import Dict
import json


class StreamSquareSize:
    def __init__(self, value: str):
        self._type = "StreamSquareSize" 
        self.value = value

    @staticmethod
    def S():
        return StreamSquareSize("S")

    @staticmethod
    def M():
        return StreamSquareSize("M")

    @staticmethod
    def L():
        return StreamSquareSize("L")

    @staticmethod
    def XL():
        return StreamSquareSize("XL")

    @staticmethod
    def XXL():
        return StreamSquareSize("XXL")

    @staticmethod
    def all_possibles_values():
        return {"S",
                "M",
                "L",
                "XL",
                "XXL"}

    @staticmethod
    def from_string(value):
        if value == "S": return StreamSquareSize.S()
        elif value == "M": return StreamSquareSize.M()
        elif value == "L": return StreamSquareSize.L()
        elif value == "XL": return StreamSquareSize.XL()
        elif value == "XXL": return StreamSquareSize.XXL()
        else: return None

    @staticmethod
    def is_valid(value):
        return value in StreamSquareSize.all_possibles_values()

    def is_not_s(self):
        return self.value != "S"

    def is_not_m(self):
        return self.value != "M"

    def is_not_l(self):
        return self.value != "L"

    def is_not_xl(self):
        return self.value != "XL"

    def is_not_xxl(self):
        return self.value != "XXL"

    def is_s(self):
        return self.value == "S"

    def is_m(self):
        return self.value == "M"

    def is_l(self):
        return self.value == "L"

    def is_xl(self):
        return self.value == "XL"

    def is_xxl(self):
        return self.value == "XXL"

    def to_json(self):
        return json.dumps(self.value)

    def __to_remote_json(self):
        return json.dumps(self.value)

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "StreamSquareSize(value=" + self.value + ")"

    def __str__(self):
        return self.value


import base64
import hashlib
import hmac
import json
import requests
from typing import Optional
from sidemash_sdk.Sdk import Sdk
from sidemash_sdk.Auth import Auth
from sidemash_sdk.HttpCallError import HttpCallError
from sidemash_sdk.SdmRequest import SdmRequest
from sidemash_sdk.RestCollection import RestCollection


class Http:

    @staticmethod
    def post(path: str, body: Optional[dict], query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.__call(path, "POST", body, auth, query_string, converter, headers)

    @staticmethod
    def list_all(path: str, query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.get(path, auth, query_string, lambda json: RestCollection.from_json(json, converter), headers)

    @staticmethod
    def get(path: str, query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.__call(path, "GET", None, auth, query_string, converter, headers)

    @staticmethod
    def patch(path: str, body: Optional[dict], query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.__call(path, "PATCH", body, auth, query_string, converter, headers)

    @staticmethod
    def put(path: str, body: Optional[dict], query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.__call(path, "PUT", body, auth, query_string, converter, headers)

    @staticmethod
    def delete(path: str, body: Optional[dict], query_string: Optional[dict], headers: Optional[dict], converter, auth: Auth):
        return Http.__call(path, "DELETE", body, auth, query_string, converter, headers)

    @staticmethod
    def __sign(message: str, private_key: str):
        signature = hmac.new(bytes(private_key, "utf-8"), bytes(message, "utf-8"), hashlib.sha512).digest()
        return base64.b64encode(signature).decode()

    @staticmethod
    def __sha256(message: str):
        h = hashlib.sha256(bytes(message,  "utf-8")).digest()
        return base64.b64encode(h).decode()

    @staticmethod
    def __compute_signed_headers(body: Optional[str], headers: dict, auth: Auth) -> dict:
        signed_headers = dict(
            {"Accept": "application/json", "User-Agent": "Sdk Python " + Sdk.version(), "Authorization": "Bearer " + auth.token},
            **headers,
            **{"Content-Type": "application/json"} if body is not None else {}
        )
        return signed_headers

    @staticmethod
    def __make_request(method: str, url: str, body: Optional[str], headers: dict):
        r = None
        if method == "POST":
            r = requests.post(url, data=body, headers=headers)
        elif method == "GET":
            r = requests.get(url, headers=headers)
        elif method == "PATCH":
            r = requests.patch(url, data=body, headers=headers)
        elif method == "PUT":
            r = requests.put(url, data=body, headers=headers)
        elif method == "DELETE":
            r = requests.delete(url, headers=headers)
        return r


    @staticmethod
    def __call(path: str, method: str, body: Optional[dict], auth: Auth, query_string: Optional[str], converter, headers: dict):
        body_str = json.dumps(body) if body is not None else None
        url = Sdk.base_url() + path + (query_string if query_string is not None else "")
        signed_headers = Http.__compute_signed_headers(body, headers if headers is not None else {}, auth)
        sdm_request = SdmRequest(signed_headers,
                                 method,
                                 path,
                                 query_string,
                                 body_hash=Http.__sha256(body_str) if body_str is not None else None)
        h = signed_headers
        h["X-Sdm-Nonce"] = str(sdm_request.nonce)
        h["X-Sdm-SignedHeaders"] = ",".join([header_name for header_name in list(signed_headers.keys())])
        h["X-Sdm-Signature"] = "SHA512 " + Http.__sign(sdm_request.to_message(), auth.private_key)
        response = Http.__make_request(method, url, body_str, h)
        if response.status_code >= 300:
            raise HttpCallError(response.status_code, response.reason, response.json())
        elif response.status_code == 204:
            return None
        else:
            return converter(response.json())



from typing import Dict
import json


class Play:
    def __init__(self, todo: int):
        self._type = "Play" 
        self.todo = todo

    @staticmethod
    def _type(): 
        return "Play"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Play.from_dict(value)

    def __to_remote_dict(self):
        tuples = [('todo', self.todo)]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([('todo', self.todo)])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return Play(d["todo"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return Play(d["todo"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "Play(todo=" + str(self.todo) + ")"


from sidemash_sdk.Timestamp import Timestamp
from typing import Dict
import json


class UTCDateTime:
    def __init__(self, iso8601: str, timestamp: Timestamp):
        self._type = "UTCDateTime" 
        self.iso8601 = iso8601
        self.timestamp = timestamp

    @staticmethod
    def _type(): 
        return "UTCDateTime"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return UTCDateTime.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('iso8601', self.iso8601),
            ('timestamp', self.timestamp.__to_remote_dict())
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('iso8601', self.iso8601),
            ('timestamp', self.timestamp.to_dict())
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return UTCDateTime(d["iso8601"],
                           Timestamp.__from_remote_dict(d["timestamp"]))

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return UTCDateTime(d["iso8601"],
                           Timestamp.from_dict(d["timestamp"]))

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("UTCDateTime(iso8601=" + self.iso8601 + 
                            ", timestamp=" + repr(self.timestamp) + ")")


from sidemash_sdk.Auth import Auth
from sidemash_sdk.CreateStreamSquareForm import CreateStreamSquareForm
from sidemash_sdk.Http import Http
from sidemash_sdk.ListForm import ListForm
from sidemash_sdk.RestCollection import RestCollection
from sidemash_sdk.Sdk import Sdk
from sidemash_sdk.StreamSquare import StreamSquare
from sidemash_sdk.UpdateStreamSquareForm import UpdateStreamSquareForm


class StreamSquareService:
    def __init__(self, auth: Auth):
        self._type = "StreamSquareService" 
        self.auth = auth

    def create(self, form: CreateStreamSquareForm):
        return Http.post("/" + Sdk.version() + "/stream-squares", form.to_json(), "", {}, StreamSquare.from_json, self.auth)

    def get(self, id: str):
        return Http.get("/" + Sdk.version() + "/stream-squares/" + id, "", {}, StreamSquare.from_json, self.auth)

    def list_all(self, form: ListForm = None):
        qs = ListForm.empty().to_query_string() if form is None else form.to_query_string()
        return Http.list_all("/" + Sdk.version() + "/stream-squares", qs, {}, lambda json: RestCollection.from_json(json, StreamSquare.from_json), self.auth)

    def update(self, form: UpdateStreamSquareForm):
        return Http.patch("/" + Sdk.version() + "/stream-squares/" + form.id, form.to_json(), "", {}, StreamSquare.from_json, self.auth)

    def delete(self, id: str):
        Http.delete("/" + Sdk.version() + "/stream-squares/" + id, None, "", {}, None, self.auth)

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "StreamSquareService(auth=" + repr(self.auth) + ")"


from typing import Dict
from typing import Set
import json


class UpdateStreamSquareForm:
    def __init__(self, id: str, remove: Set[str], set: Dict[str, any]):
        self._type = "UpdateStreamSquareForm" 
        self.id     = id
        self.remove = remove
        self.set    = set

    @staticmethod
    def by_id(id):
        return UpdateStreamSquareForm(id, set(), dict())

    def remove_description(self):
        remove = self.remove.union({'description'})
        return UpdateStreamSquareForm(self.id, remove, self.set)

    def remove_foreign_data(self):
        remove = self.remove.union({'foreignData'})
        return UpdateStreamSquareForm(self.id, remove, self.set)

    def set_is_elastic(self, is_elastic):
        s = dict(self.set, **{'isElastic': is_elastic})
        return UpdateStreamSquareForm(self.id, self.remove, s)

    def set_size(self, size):
        s = dict(self.set, **{'size': str(size)})
        return UpdateStreamSquareForm(self.id, self.remove, s)

    def set_hook(self, hook):
        s = dict(self.set, **{'hook': hook.to_dict()})
        return UpdateStreamSquareForm(self.id, self.remove, s)

    def set_description(self, description):
        if description is None: 
            return self
        s = dict(self.set, **{'description': description})
        return UpdateStreamSquareForm(self.id, self.remove, s)

    def set_foreign_data(self, foreign_data):
        if foreign_data is None: 
            return self
        s = dict(self.set, **{'foreignData': foreign_data})
        return UpdateStreamSquareForm(self.id, self.remove, s)

    def __to_remote_dict(self):
        tuples = [
            self.remove,
            ('set', self.set.__to_remote_dict())
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('id', self.id),
            self.remove,
            ('set', self.set.to_dict())
        ])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("UpdateStreamSquareForm(id=" + self.id + 
                                       ", remove=" + repr(self.remove) + 
                                       ", set=" + repr(self.set) + ")")


from sidemash_sdk.Pagination import Pagination
from typing import Dict
from typing import List
from typing import TypeVar
import json

T = TypeVar('T')


class RestCollection:
    def __init__(self, url: str, pagination: Pagination, items: List[T]):
        self._type = "RestCollection" 
        self.url = url
        self.pagination = pagination
        self.items = items

    @staticmethod
    def _type(): 
        return "RestCollection"

    @staticmethod
    def from_json(js: str, item_converter):
        value = json.loads(js)
        items = list(map(lambda item: item_converter(item), value['items']))
        return RestCollection.from_dict({
            'url': value['url'],
            'pagination': value['pagination'],
            'items': items,
        })

    def __to_remote_dict(self):
        tuples = [
            ('url', self.url),
            ('pagination', self.pagination.__to_remote_dict()),
            ('items', list(map(lambda el: el.__to_remote_dict(), self.items)))
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('url', self.url),
            ('pagination', self.pagination.to_dict()),
            ('items', list(map(lambda el: el.to_dict(), self.items)))
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return RestCollection(d["url"],
                              Pagination.__from_remote_dict(d["pagination"]),
                              list(map(lambda el: el.__from_remote_dict(el), d["items"])))

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return RestCollection(d["url"],
                              Pagination.from_dict(d["pagination"]),
                              list(map(lambda el: el.from_dict(el), d["items"])))

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("RestCollection(url=" + self.url + 
                               ", pagination=" + repr(self.pagination) + 
                               ", items=" + repr(self.items) + ")")



class HttpCallError(Exception):

    def __init__(self, status_code: int, status_message: str, body: str):
        self.status_code = status_code
        self.status_message = status_message
        self.body = body



from typing import Dict
import json


class InstanceStatus:
    def __init__(self, value: str):
        self.value = value

    @staticmethod
    def Initializing():
        return InstanceStatus("Initializing")

    @staticmethod
    def Running():
        return InstanceStatus("Running")

    @staticmethod
    def Restarting():
        return InstanceStatus("Restarting")

    @staticmethod
    def Updating():
        return InstanceStatus("Updating")

    @staticmethod
    def Maintaining():
        return InstanceStatus("Maintaining")

    @staticmethod
    def PartiallyAvailable():
        return InstanceStatus("PartiallyAvailable")

    @staticmethod
    def NotAvailable():
        return InstanceStatus("NotAvailable")

    @staticmethod
    def Terminating():
        return InstanceStatus("Terminating")

    @staticmethod
    def Terminated():
        return InstanceStatus("Terminated")

    @staticmethod
    def all_possibles_values():
        return {"Initializing",
                "Running",
                "Restarting",
                "Updating",
                "Maintaining",
                "PartiallyAvailable",
                "NotAvailable",
                "Terminating",
                "Terminated"}

    @staticmethod
    def from_string(value):
        if value == "Initializing": return InstanceStatus.Initializing()
        elif value == "Running": return InstanceStatus.Running()
        elif value == "Restarting": return InstanceStatus.Restarting()
        elif value == "Updating": return InstanceStatus.Updating()
        elif value == "Maintaining": return InstanceStatus.Maintaining()
        elif value == "PartiallyAvailable": return InstanceStatus.PartiallyAvailable()
        elif value == "NotAvailable": return InstanceStatus.NotAvailable()
        elif value == "Terminating": return InstanceStatus.Terminating()
        elif value == "Terminated": return InstanceStatus.Terminated()
        else: return None

    @staticmethod
    def is_valid(value):
        return value in InstanceStatus.all_possibles_values()

    def is_not_initializing(self):
        return self.value != "Initializing"

    def is_not_running(self):
        return self.value != "Running"

    def is_not_restarting(self):
        return self.value != "Restarting"

    def is_not_updating(self):
        return self.value != "Updating"

    def is_not_maintaining(self):
        return self.value != "Maintaining"

    def is_not_partially_available(self):
        return self.value != "PartiallyAvailable"

    def is_not_not_available(self):
        return self.value != "NotAvailable"

    def is_not_terminating(self):
        return self.value != "Terminating"

    def is_not_terminated(self):
        return self.value != "Terminated"

    def is_initializing(self):
        return self.value == "Initializing"

    def is_running(self):
        return self.value == "Running"

    def is_restarting(self):
        return self.value == "Restarting"

    def is_updating(self):
        return self.value == "Updating"

    def is_maintaining(self):
        return self.value == "Maintaining"

    def is_partially_available(self):
        return self.value == "PartiallyAvailable"

    def is_not_available(self):
        return self.value == "NotAvailable"

    def is_terminating(self):
        return self.value == "Terminating"

    def is_terminated(self):
        return self.value == "Terminated"

    def to_json(self):
        return json.dumps(self.value)

    def __to_remote_json(self):
        return json.dumps(self.value)

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "InstanceStatus(value=" + self.value + ")"

    def __str__(self):
        return self.value


from sidemash_sdk.SecureAndNonSecure import SecureAndNonSecure
from typing import Dict
import json


class PublishRtmp:
    def __init__(self,
                 stream_key_prefix: str,
                 ip: SecureAndNonSecure,
                 domain: SecureAndNonSecure):
        self._type = "PublishRtmp" 
        self.stream_key_prefix = stream_key_prefix
        self.ip = ip
        self.domain = domain

    @staticmethod
    def _type(): 
        return "PublishRtmp"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return PublishRtmp.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('streamKeyPrefix', self.stream_key_prefix),
            ('ip', self.ip.__to_remote_dict()),
            ('domain', self.domain.__to_remote_dict())
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('stream_key_prefix', self.stream_key_prefix),
            ('ip', self.ip.to_dict()),
            ('domain', self.domain.to_dict())
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return PublishRtmp(d["streamKeyPrefix"],
                           SecureAndNonSecure.__from_remote_dict(d["ip"]),
                           SecureAndNonSecure.__from_remote_dict(d["domain"]))

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return PublishRtmp(d["stream_key_prefix"],
                           SecureAndNonSecure.from_dict(d["ip"]),
                           SecureAndNonSecure.from_dict(d["domain"]))

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("PublishRtmp(stream_key_prefix=" + self.stream_key_prefix + 
                            ", ip=" + repr(self.ip) + 
                            ", domain=" + repr(self.domain) + ")")


from sidemash_sdk.UTCDateTime import UTCDateTime
from typing import Dict
from typing import Optional
import json


class Pagination:
    def __init__(self,
                 self_url: str,
                 prev_url: Optional[str],
                 next_url: Optional[str],
                 started_time: UTCDateTime,
                 nb_items_on_this_page: int,
                 page: int,
                 nb_items_per_page: int):
        self._type = "Pagination" 
        self.self_url = self_url
        self.prev_url = prev_url
        self.next_url = next_url
        self.started_time = started_time
        self.nb_items_on_this_page = nb_items_on_this_page
        self.page = page
        self.nb_items_per_page = nb_items_per_page

    @staticmethod
    def _type(): 
        return "Pagination"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Pagination.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('selfUrl', self.self_url),
            ('prevUrl', self.prev_url),
            ('nextUrl', self.next_url),
            ('startedTime', self.started_time.__to_remote_dict()),
            ('nbItemsOnThisPage', self.nb_items_on_this_page),
            ('page', self.page),
            ('nbItemsPerPage', self.nb_items_per_page)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('self_url', self.self_url),
            ('prev_url', self.prev_url),
            ('next_url', self.next_url),
            ('started_time', self.started_time.to_dict()),
            ('nb_items_on_this_page', self.nb_items_on_this_page),
            ('page', self.page),
            ('nb_items_per_page', self.nb_items_per_page)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return Pagination(d["selfUrl"],
                          d["prevUrl"] if "prevUrl" in d else None,
                          d["nextUrl"] if "nextUrl" in d else None,
                          UTCDateTime.__from_remote_dict(d["startedTime"]),
                          d["nbItemsOnThisPage"],
                          d["page"],
                          d["nbItemsPerPage"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return Pagination(d["self_url"],
                          d["prev_url"] if "prev_url" in d else None,
                          d["next_url"] if "next_url" in d else None,
                          UTCDateTime.from_dict(d["started_time"]),
                          d["nb_items_on_this_page"],
                          d["page"],
                          d["nb_items_per_page"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("Pagination(self_url=" + self.self_url + 
                           ", prev_url=" + str(self.prev_url) + 
                           ", next_url=" + str(self.next_url) + 
                           ", started_time=" + repr(self.started_time) + 
                           ", nb_items_on_this_page=" + str(self.nb_items_on_this_page) + 
                           ", page=" + str(self.page) + 
                           ", nb_items_per_page=" + str(self.nb_items_per_page) + ")")


from typing import Dict
import json


class Auth:
    def __init__(self, token: str, private_key: str):
        self._type = "Auth" 
        self.token = token
        self.private_key = private_key

    @staticmethod
    def _type(): 
        return "Auth"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Auth.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('token', self.token),
            ('privateKey', self.private_key)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('token', self.token),
            ('private_key', self.private_key)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return Auth(d["token"],
                    d["privateKey"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return Auth(d["token"],
                    d["private_key"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("Auth(token=" + self.token + 
                     ", private_key=******" + ")")


from sidemash_sdk.Hook import Hook
from sidemash_sdk.StreamSquareSize import StreamSquareSize
from typing import Dict
from typing import Optional
import json


class CreateStreamSquareForm:
    def __init__(self,
                 is_elastic: bool,
                 size: StreamSquareSize,
                 hook: Hook,
                 description: Optional[str] = None,
                 foreign_data: Optional[str] = None,
                 play_domain_name: Optional[str] = None,
                 publish_domain_name: Optional[str] = None):
        self._type = "CreateStreamSquareForm" 
        self.is_elastic = is_elastic
        self.size = size
        self.hook = hook
        self.description = description
        self.foreign_data = foreign_data
        self.play_domain_name = play_domain_name
        self.publish_domain_name = publish_domain_name

    @staticmethod
    def _type(): 
        return "CreateStreamSquareForm"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return CreateStreamSquareForm.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('isElastic', self.is_elastic),
            ('size', str(self.size)),
            ('hook', self.hook.__to_remote_dict()),
            ('description', self.description),
            ('foreignData', self.foreign_data),
            ('playDomainName', self.play_domain_name),
            ('publishDomainName', self.publish_domain_name)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('is_elastic', self.is_elastic),
            ('size', str(self.size)),
            ('hook', self.hook.to_dict()),
            ('description', self.description),
            ('foreign_data', self.foreign_data),
            ('play_domain_name', self.play_domain_name),
            ('publish_domain_name', self.publish_domain_name)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return CreateStreamSquareForm(d["isElastic"],
                                      StreamSquareSize.from_string(d["size"]),
                                      Hook.__from_remote_dict(d["hook"]),
                                      d["description"] if "description" in d else None,
                                      d["foreignData"] if "foreignData" in d else None,
                                      d["playDomainName"] if "playDomainName" in d else None,
                                      d["publishDomainName"] if "publishDomainName" in d else None)

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return CreateStreamSquareForm(d["is_elastic"],
                                      StreamSquareSize.from_string(d["size"]),
                                      Hook.from_dict(d["hook"]),
                                      d["description"] if "description" in d else None,
                                      d["foreign_data"] if "foreign_data" in d else None,
                                      d["play_domain_name"] if "play_domain_name" in d else None,
                                      d["publish_domain_name"] if "publish_domain_name" in d else None)

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("CreateStreamSquareForm(is_elastic=" + str(self.is_elastic) + 
                                       ", size=" + repr(self.size) + 
                                       ", hook=" + repr(self.hook) + 
                                       ", description=" + str(self.description) + 
                                       ", foreign_data=" + str(self.foreign_data) + 
                                       ", play_domain_name=" + str(self.play_domain_name) + 
                                       ", publish_domain_name=" + str(self.publish_domain_name) + ")")


from typing import Dict
import json


class Timestamp:
    def __init__(self, seconds: int):
        self._type = "Timestamp" 
        self.seconds = seconds

    @staticmethod
    def _type(): 
        return "Timestamp"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Timestamp.from_dict(value)

    def __to_remote_dict(self):
        tuples = [('seconds', self.seconds)]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([('seconds', self.seconds)])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return Timestamp(d["seconds"])

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return Timestamp(d["seconds"])

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "Timestamp(seconds=" + str(self.seconds) + ")"


from typing import Dict
import json


class HttpMethod:
    def __init__(self, value: str):
        self._type = "HttpMethod" 
        self.value = value

    @staticmethod
    def GET():
        return HttpMethod("GET")

    @staticmethod
    def POST():
        return HttpMethod("POST")

    @staticmethod
    def PUT():
        return HttpMethod("PUT")

    @staticmethod
    def DELETE():
        return HttpMethod("DELETE")

    @staticmethod
    def PATCH():
        return HttpMethod("PATCH")

    @staticmethod
    def all_possibles_values():
        return {"GET",
                "POST",
                "PUT",
                "DELETE",
                "PATCH"}

    @staticmethod
    def from_string(value):
        if value == "GET": return HttpMethod.GET()
        elif value == "POST": return HttpMethod.POST()
        elif value == "PUT": return HttpMethod.PUT()
        elif value == "DELETE": return HttpMethod.DELETE()
        elif value == "PATCH": return HttpMethod.PATCH()
        else: return None

    @staticmethod
    def is_valid(value):
        return value in HttpMethod.all_possibles_values()

    def is_not_get(self):
        return self.value != "GET"

    def is_not_post(self):
        return self.value != "POST"

    def is_not_put(self):
        return self.value != "PUT"

    def is_not_delete(self):
        return self.value != "DELETE"

    def is_not_patch(self):
        return self.value != "PATCH"

    def is_get(self):
        return self.value == "GET"

    def is_post(self):
        return self.value == "POST"

    def is_put(self):
        return self.value == "PUT"

    def is_delete(self):
        return self.value == "DELETE"

    def is_patch(self):
        return self.value == "PATCH"

    def to_json(self):
        return json.dumps(self.value)

    def __to_remote_json(self):
        return json.dumps(self.value)

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "HttpMethod(value=" + self.value + ")"

    def __str__(self):
        return self.value


from sidemash_sdk.PublishRtmp import PublishRtmp
from typing import Dict
import json


class Publish:
    def __init__(self, rtmp: PublishRtmp):
        self._type = "Publish" 
        self.rtmp = rtmp

    @staticmethod
    def _type(): 
        return "Publish"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return Publish.from_dict(value)

    def __to_remote_dict(self):
        tuples = [('rtmp', self.rtmp.__to_remote_dict())]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([('rtmp', self.rtmp.to_dict())])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return Publish(PublishRtmp.__from_remote_dict(d["rtmp"]))

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return Publish(PublishRtmp.from_dict(d["rtmp"]))

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return "Publish(rtmp=" + repr(self.rtmp) + ")"


from typing import Dict
from typing import Optional
import json


class ListForm:
    def __init__(self,
                 where: Optional[str] = None,
                 limit: Optional[int] = None,
                 order_by: Optional[str] = None):
        self._type = "ListForm" 
        self.where = where
        self.limit = limit
        self.order_by = order_by

    @staticmethod
    def _type(): 
        return "ListForm"

    @staticmethod
    def from_json(js: str):
        value = json.loads(js)
        return ListForm.from_dict(value)

    def __to_remote_dict(self):
        tuples = [
            ('where', self.where),
            ('limit', self.limit),
            ('orderBy', self.order_by)
        ]
        return dict(t for t in tuples if t[1] is not None)

    def to_dict(self):
        return dict([
            ('where', self.where),
            ('limit', self.limit),
            ('order_by', self.order_by)
        ])

    @staticmethod
    def __from_remote_dict(d: Dict[str, any]):
        return ListForm(d["where"] if "where" in d else None,
                        d["limit"] if "limit" in d else None,
                        d["orderBy"] if "orderBy" in d else None)

    @staticmethod
    def from_dict(d: Dict[str, any]):
        return ListForm(d["where"] if "where" in d else None,
                        d["limit"] if "limit" in d else None,
                        d["order_by"] if "order_by" in d else None)

    def to_json(self):
        return json.dumps(self.to_dict())

    def __to_remote_json(self):
        return json.dumps(self.__to_remote_dict())

    @staticmethod
    def empty():
        return ListForm(None, None, None)

    def to_query_string(self) -> str:
        if self.where is None and self.limit is None and self.order_by is None :
            return "" 
        else:
            return "?" + "&".join(t[0] + "=" + t[1] for t in [self.__to_remote_dict()] if t[1] is not None)

    def to_string(self):
        return self.__repr__()

    def __repr__(self):
        return ("ListForm(where=" + str(self.where) + 
                         ", limit=" + str(self.limit) + 
                         ", order_by=" + str(self.order_by) + ")")



class Sdk:
    @staticmethod
    def version():
        return "v1.0"

    @staticmethod
    def host():
        return "dev-api.sidemash.io"

    @staticmethod
    def base_url(): 
        return "https://" + Sdk.host()




from sidemash_sdk.Auth import Auth
from sidemash_sdk.Client import Client
from sidemash_sdk.StreamSquareSize import StreamSquareSize
from sidemash_sdk.UpdateStreamSquareForm import UpdateStreamSquareForm

auth = Auth(token= "1234", private_key= "****")
sdm = Client(auth)
s = StreamSquareSize
print(s)
print(s)
# form = UpdateStreamSquareForm
#    .by_id("01")
#    .remove_description()
#    .set_foreign_data("OK")
#    .set_is_elastic(True)
# sdm.stream_square().update(form)

#form = form.set_size(StreamSquareSize.S())

#sdm = Client(Auth(token="1234", private_key="****"))
#domain1 = sdm.domain.create(CreateDomainForm(name="ville-lille.com", purpose="Publish", description="My example domain name", foreign_data = None))
#domain2 = sdm.domain.getById("AZERTY")
#print(domain1)

# from sidemash_sdk.model.StreamSquare import StreamSquare
# from sidemash_sdk.form.CreateStreamSquareForm import CreateStreamSquareForm
# sdm.streamSquare.create(CreateStreamSquareForm(size=StreamSquare.Size.M, is_elastic=False))
